/**
 * Concrete {@linkplain org.junit.platform.engine.DiscoverySelector selectors} and
 * {@linkplain org.junit.platform.engine.DiscoveryFilter filters} to be used in
 * {@linkplain org.junit.platform.engine.EngineDiscoveryRequest discovery requests}.
 */

package org.junit.platform.engine.discovery;
