/**
 * Annotations for configuring a <em>test suite</em> on the JUnit Platform.
 *
 * <h2>JUnit 4 Suite Support</h2>
 * <p>Test suites can be run on the JUnit Platform in a JUnit 4 environment via
 * {@code @RunWith(JUnitPlatform.class)} with the {@code junit-platform-runner}.
 *
 * <h2>JUnit 5 Suite Support</h2>
 * <p>Test suites can be run on the JUnit Platform in a JUnit 5 environment via
 * {@link Suite @Suite} with the {@code junit-platform-suite-engine}.
 */

package org.junit.platform.suite.api;
